<?php  

require_once(ParticleTraffic_PATH.'lib/db.php');
require_once(ParticleTraffic_PATH.'lib/visit.php');
require_once(ParticleTraffic_PATH.'lib/hit.php');
require_once(ParticleTraffic_PATH.'lib/cached_month.php');
require_once(ParticleTraffic_PATH.'lib/cached_day.php');
require_once(ParticleTraffic_PATH.'lib/tools.php');
require_once(ParticleTraffic_PATH.'lib/display.php');
//require_once(ParticleTraffic_PATH.'lib/installer.php');
//require_once(ParticleTraffic_PATH.'lib/authentication.php');

class ParticleTrafficTracker { 

	protected $dao_loader; 
	protected $db;
	protected $book_id=0; //book_id is your siteIdNumber
	
	public static $visits_table='TonyStatsVisits';
	public static $hits_table='TonyStatsHits';
	public static $cache_table='TonyStatsCachedMonths';
	public static $cache_days_table='TonyStatsCachedDays';	
	
	
	//the amount of time spent idle in seconds before new page views will create a new visit
	static public $visitTimeout=1800;
	
	public static function quickTrack( $options=array() ){		
		$statsTracker = new ParticleTrafficTracker( intval($options['siteIdNumber']) );
		$statsTracker->trackHit( $options ); 
	}
	
	public static function printTrackingJS($siteIdNumber=0){
		echo '<script src="'.ParticleTraffic_URL_DIR.'ParticleTraffic.services.php?mode=tracker_js&siteIdNumber='.intval($siteIdNumber).'" type="text/javascript"></script> '."\n";
	}

	public function __construct($siteIdNumber=0){ 
		$this->db = ParticleTrafficDB::get_instance(); 
		$this->book_id=intval($siteIdNumber);
	} 
	
	//a hit triggered from the browser site
	public function browserSideHit( $options=array() ){
		$visit=$this->getThisVisit();
		$hitVO = new ParticleTrafficHit();	
		$hitVO->set_traffic_visits_id( intval($visit->traffic_visit_id) );
		$url=ParticleTrafficTools::simpleText( $_REQUEST['url'] );
		$hitVO->set_page_url($url); 
		$short_url=(strstr($url,'?')) ? substr($url,0,strpos($url,'?')) : $url;	
		$hitVO->set_page_url_short( $short_url ); 		
		$hitVO->set_page_title( strip_tags( $_REQUEST['t'] ) ); 		
		$hitVO->create();		
	}	
	
	//serverSideHit
	public function trackHit($options=array()){ 
	
		//if this is enabled, then don't create a new visit if one doesn't exist.  used on feeds. 
		$createVisit = ($options['realUserCheck'])?0:1;
		
		$visit=$this->getThisVisit( $createVisit ); 
		if(!$visit) return false;  
		$hitVO = new ParticleTrafficHit();	
		$hitVO->set_traffic_visits_id(intval($visit->traffic_visit_id));
		$url=$_SERVER['REQUEST_URI'];
		if( strlen($options['url']) ) $url=ParticleTrafficTools::simpleText($options['url']);
		$hitVO->set_page_url($url); 
		$short_url=(strstr($url,'?')) ? substr($url,0,strpos($url,'?')) : $url;	
		$hitVO->set_page_url_short( $short_url ); 		
		$hitVO->set_page_title( strip_tags($options['title']) ); 
		$hitVO->set_event_value( strip_tags($options['eventValue']) ); 
		$hitVO->set_event_only( intval($options['eventOnly']) ); 
		$hitVO->create();
	} 
	
	static public function pageUnload(){  
		$statsTracker = new ParticleTrafficTracker( intval($options['siteIdNumber']) );
		$visit=$statsTracker->getThisVisit( 0 );
		if($visit){ 
			$visit->set_last_page_unloaded( time() );
			$visit->save();
			die;
		}
	} 
	
	//for ignore certain groups 
	public function getConfigSettings(){
		if(is_array($this->trackingSettings)) return $this->trackingSettings; 
		$this->trackingSettings = unserialize(Config::get('TONY_STATS_TRACKING_SETTINGS')); 
		if(!is_array($this->trackingSettings)) $this->trackingSettings=array();	
		return $this->trackingSettings;
	}
	
	public function getVisitId(){
		$session_id=session_id();
		if( !strlen($session_id) )
			throw new Exception('You must start a php session with session_start() for ParticleTraffic to work');
		return intval($this->book_id).'_'.$_SERVER['REMOTE_ADDR'].'_'.$session_id;
	}
	
	public function getThisVisit($withCreate=1,$user_id=-1){ 
		$browserInfo=$this->getBrowserInfo();
		$trackingSettings = $this->getConfigSettings();
		
		$usersGroups=array();
		
		if($user_id==-1){
			
			$expire = time()+(60*60*24); 
			
			$u = new User();	
			
			$user_id = intval($u->uID);  
			
			//save user id to cookie, so we know who they are next time 
			if($user_id){
				setcookie('stats_uID', $user_id , $expire);
				
			//load user from cookied user id 	
			}elseif( isset($_COOKIE['stats_uID']) && intval($_COOKIE['stats_uID']) ) { 
				$u = User::getByUserID( intval($_COOKIE['stats_uID']) ); 
				$user_id = intval($u->uID);  
			} 
			
			//get cached groups
			if( (intval($user_id) || intval($trackingSettings['ignoreUnlogged'])) && isset($_COOKIE['stats_gIDs']) && intval($_COOKIE['stats_config_timestamp'])==intval($trackingSettings['timestamp']) ){
				//echo 'TEST';
				$usersGroups=explode(',',$_COOKIE['stats_gIDs']); 
				
			//or else load and cache groups 	
			}elseif( intval($user_id)>0  ){  
				$tmpUsrGroups = $u->getUserGroups(); 
				if(is_array($tmpUsrGroups)) foreach( $tmpUsrGroups as $gID=>$gName) $usersGroups[]=intval($gID); 
				   
				setcookie('stats_gIDs', join(',',$usersGroups), $expire);
				setcookie('stats_config_timestamp', intval($trackingSettings['timestamp']), $expire);
			}
		}   
		
		//var_dump($usersGroups);
		
		//is this user in one of the ignored groups? 
		if( is_array($usersGroups) && is_array($trackingSettings['gIDs'])){ 
			foreach($usersGroups as $gID){
				if(intval($gID) && in_array(intval($gID),$trackingSettings['gIDs'])) return false; 
			}
		} 
		
		//get by 
		$session_id=$this->getVisitId();
		$visit=ParticleTrafficVisit::retrieveBySession( $session_id ); 
		
		$userVisitNumber = intval($_COOKIE['stats_user_visit_number']); 
		
		$cookies_off_id=intval($this->book_id).'_'.ParticleTrafficTools::simpleText($_SERVER['REMOTE_ADDR']).'_'.ParticleTrafficTools::simpleText($browserInfo['platform']).'_'.ParticleTrafficTools::simpleText($browserInfo['browser']).'_'.$browserInfo['majorVersion'];
		if(!$visit){
			//visit not found - cookies might be disabled so check by just ip now 
			$visit = ParticleTrafficVisit::retrieveByCookiesOffId($cookies_off_id); 
			if(is_object($visit) && !$userVisitNumber) $userVisitNumber = $visit->get_returning_visit_count(); 
		}
		
		//is this visit still active? 
		if( $visit && !$visit->isVisitActive() ) $visit=NULL;
			
		if($visit){ 
			if(!$visit->users_id && $user_id>0){
				$visit->set_users_id($user_id); 						
				$visit->save();
			}
		}else{
			if(!$withCreate) return false;
			
			$userVisitNumber++; 
			setcookie('stats_user_visit_number', $userVisitNumber );
			
			$visit=new ParticleTrafficVisit();	
			$visit->set_session_id($session_id); 
			$visit->set_cookies_off_id($cookies_off_id);
			$visit->set_book_id(intval($this->book_id)); 
			$visit->set_users_id($user_id);
			$visit->set_ip($_SERVER['REMOTE_ADDR']);
			$visit->set_host_name($this->getShortRemoteHostName());		
			if( strlen($_REQUEST['ref']) && $_REQUEST['ref']!=$_REQUEST['url'] && !stristr($_REQUEST['ref'],BASE_URL))	
				$visit->set_referrer_url(str_ireplace('www.','',$_REQUEST['ref']) ); 
			//elseif(!stristr($_SERVER['HTTP_REFERER'],$_SERVER["SERVER_NAME"])) 
			//	$visit->set_referrer_url(str_ireplace('www.','',$_SERVER['HTTP_REFERER']) ); 
			if($_REQUEST['ref'] && $_REQUEST['ref']!=$_REQUEST['url'])
				$visit->set_search_terms(trim($this->getSearchTerms())); 
			$visit->set_platform($browserInfo['platform']);
			$visit->set_browser($browserInfo['browser']);
			$visit->set_browser_version($browserInfo['majorVersion']); 
			$visit->set_returning_visit_count( $userVisitNumber );  
			if($_REQUEST['fv']) $visit->set_flash_version(intval($_REQUEST['fv']));
			if($_REQUEST['wx'] && $_REQUEST['wy']) $visit->set_resolution(intval($_REQUEST['wx']).' x '.intval($_REQUEST['wy']));
			$visit->save();
			
		}
		return $visit;
	}	
	
	public function getSearchTerms(){
		$url=$_REQUEST['ref']; 
		$refUrlParts=explode('?',$url);
		$refUrlBase=$refUrlParts[0];
		if(stristr($refUrlBase,$_SERVER["SERVER_NAME"])) return ''; 
		if( strpos($url,'?')<=0 ) return ''; 
		$qStr=substr($url,strpos($url,'?')+1); 
		$keyVals=explode('&',$qStr);
		foreach($keyVals as $keyValPair){ 
			if(!strstr($keyValPair,'=')) continue;
			$parts=explode('=',$keyValPair); 
			if(strtolower($parts[0])=='q') return urldecode($parts[1]);
			if(strtolower($parts[0])=='p' && stristr($_REQUEST['ref'],'yahoo')) return urldecode($parts[1]); 
		}
		return '';
	}

	public function getShortRemoteHostName(){
		$ip=$_SERVER['REMOTE_ADDR'];
		$dashed_ip=str_replace('.','-',$ip);
		$host=gethostbyaddr($ip);
		if($host==$ip) return '';
		return str_replace(array($dashed_ip.'.',$ip.'.'),'',$host);
	} 
	
	function getBrowserInfo() {
		
		$browser      = "unrecognized";
		$majorVersion = "0";
		$minorVersion = "0";
		$fullVersion  = "0.0";
		$platform     = 'unrecognized';	
	
		// Note: An excellent article on browser IDs can be found at
		// http://www.zytrax.com/tech/web/browser_ids.htm
	
		$SUPERCLASS_NAMES = "gecko,mozilla,mosaic,webkit";
		$SUPERCLASS_REGX  = "(?:".str_replace(",", ")|(?:", $SUPERCLASS_NAMES).")";
	
		$SUBCLASS_NAMES   = "opera,msie,firefox,chrome,safari";
		$SUBCLASS_REGX    = "(?:".str_replace(",", ")|(?:", $SUBCLASS_NAMES).")";
	
		$userAgent    = strtolower($_SERVER['HTTP_USER_AGENT']); 
		
		// > IE11+ user agent 
		//$userAgent    = 'Mozilla/5.0 (Windows NT 6.3; Trident/7.0; rv:11.1.1) like Gecko'; 
		// < IE10- user agent 
		//$userAgent    = 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0)'; 
	
		$found = preg_match("/(?P<browser>".$SUBCLASS_REGX.")(?:\D*)(?P<majorVersion>\d*)(?P<minorVersion>(?:\.\d*)*)/i",$userAgent, $matches);
		if (!$found) {
			$found = preg_match("/(?P<browser>".$SUPERCLASS_REGX.")(?:\D*)(?P<majorVersion>\d*)(?P<minorVersion>(?:\.\d*)*)/i",$userAgent, $matches);
		}
	
		if ($found) {
			$browser      = $matches["browser"];
			$majorVersion = $matches["majorVersion"];
			$minorVersion = $matches["minorVersion"];
			$fullVersion  = $matches["majorVersion"].$matches["minorVersion"];
			if ($browser == "safari") {
				if (preg_match("/version\/(?P<majorVersion>\d*)(?P<minorVersion>(?:\.\d*)*)/i",$userAgent, $matches)){
					$majorVersion = $matches["majorVersion"];
					$minorVersion = $matches["minorVersion"];
					$fullVersion  = $majorVersion.".".$minorVersion;
				}
			} 

			//IE11+ 
			if( stristr($userAgent,"Trident") && $browser == "Mozilla" ){
			  preg_match('/Trident\/\d{1,2}.\d{1,2}; rv:([0-9\.]*)/', $userAgent, $IE_matches);
			  if (count($IE_matches)>1){
				  $browser = 'MSIE';
				  $versionParts = explode(".", $IE_matches[1] , 2) ;  
				  $majorVersion = intval($versionParts[0]); 
				  $minorVersion = ( count($versionParts)>1 ) ? intval($versionParts[1]) : 0; 
				  $fullVersion  = $majorVersion.".".$minorVersion;
			  }
			}
		}  
		
		if( stripos($userAgent, 'windows') !== false || strpos($userAgent, 'win32') ) {
			$platform = 'Windows';
		} else if( stripos($userAgent, 'iPad') !== false ) {
			$platform = 'iPad';
		} else if( stripos($userAgent, 'iPod') !== false ) {
			$platform = 'iPod';
		} else if( stripos($userAgent, 'iPhone') !== false ) {
			$platform = 'iPhone';
		} elseif( stripos($userAgent, 'mac') !== false ) {
			$platform = 'Mac';
		} elseif( stripos($userAgent, 'android') !== false ) {
			$platform ='Android';
		} elseif( stripos($userAgent, 'linux') !== false ) {
			$platform = 'Linux';
		} else if( stripos($userAgent, 'Nokia') !== false ) {
			$platform = 'Nokia';
		} else if( stripos($userAgent, 'BlackBerry') !== false ) {
			$platform = 'BlackBerry';
		} elseif( stripos($userAgent,'FreeBSD') !== false ) {
			$platform = 'FreeBSD';
		} elseif( stripos($userAgent,'OpenBSD') !== false ) {
			$platform = 'OpenBSD';
		} elseif( stripos($userAgent,'NetBSD') !== false ) {
			$platform = 'NetBSD';
		} elseif( stripos($userAgent, 'OpenSolaris') !== false ) {
			$platform = 'OpenSolaris';
		} elseif( stripos($userAgent, 'SunOS') !== false ) {
			$platform = 'SunOS';
		} elseif( stripos($userAgent, 'OS\/2') !== false ) {
			$platform = 'OS2';
		} elseif( stripos($userAgent, 'BeOS') !== false ) {
			$platform = 'BeOS';
		} elseif( stripos($userAgent, 'win') !== false ) {
			$platform = 'Windows';
		}
	
		return array(
			"browser"      => $browser,
			"majorVersion" => $majorVersion,
			"minorVersion" => $minorVersion,
			"fullVersion"  => $fullVersion,
			"platform"     => $platform,
			"userAgent"    => $userAgent);
	}	
	
	/*
	public function saveBrowserInfo(){
		$visit=$this->getThisVisit(0);
		if(!$visit) return false; 
		//$visit->set_platform($_REQUEST['p']);
		//$visit->set_browser($_REQUEST['b']);
		//$visit->set_browser_version($_REQUEST['bv']);
		$visit->set_flash_version(intval($_REQUEST['fv']));
		$visit->set_resolution(intval($_REQUEST['wx']).' x '.intval($_REQUEST['wy']));
		$visit->save();
	}
	*/
	
	protected function cleanup(){
		if( date('i')!=58 ) return;
		$sql='DELETE FROM traffic_hits WHERE created<'.(date('U')-(3600*24*365) ); 
		$this->db->query($sql);
		$sql='DELETE FROM traffic_visits WHERE created<'.(date('U')-(3600*24*365) ); 
		$this->db->query($sql); 		
	}	

} 

?>